@@ -1199,26 +1199,22 @@ dav_client_discover_iterate (GTask *task)
       g_ptr_array_sort (discover->services, services_sort_func);
       g_task_return_pointer (task,
                              g_steal_pointer (&discover->services),
-                             g_object_unref);
+                             (GDestroyNotify) g_ptr_array_unref);
     }
 }
 
 static gboolean
 dav_client_discover_preconfig (DiscoverData *discover,
-                               const char   *uri)
+                               GUri         *uri)
 {
-  g_autoptr (GUri) guri = NULL;
   const char *host = NULL;
   const char *base_domain = NULL;
 
   g_assert (discover != NULL);
 
-  guri = g_uri_parse (uri, G_URI_FLAGS_NONE, NULL);
-  if (guri == NULL)
-    return FALSE;
-
-  host = g_uri_get_host (guri);
-  base_domain = soup_tld_get_base_domain (host, NULL);
+  host = g_uri_get_host (uri);
+  if (host != NULL)
+    base_domain = soup_tld_get_base_domain (host, NULL);
 
   if (g_strcmp0 (host, "fastmail.com") == 0
       || g_strcmp0 (base_domain, "fastmail.com") == 0)
@@ -1334,7 +1330,7 @@ goa_dav_client_discover (GoaDavClient        *self,
 
   /* Check if the host can be preconfigured without lookup.
    */
-  if (dav_client_discover_preconfig (discover, uri))
+  if (dav_client_discover_preconfig (discover, guri))
     {
       dav_client_discover_iterate (task);
       return;
