@@ -71,7 +71,6 @@ typedef struct
 typedef struct
 {
   GCancellable *cancellable;
-  GCancellable *req_cancellable;
   GError *error;
   SoupMessage *msgs[2];
   SoupSession *session;
@@ -101,7 +100,6 @@ ews_client_autodiscover_data_free (gpointer user_data)
       g_object_unref (data->cancellable);
     }
 
-  g_clear_object (&data->req_cancellable);
   g_clear_error (&data->error);
 
   xmlOutputBufferClose (data->buf);
@@ -177,21 +175,13 @@ ews_client_autodiscover_cancelled_cb (GCancellable *cancellable, gpointer user_d
 static gboolean
 ews_client_autodiscover_parse_protocol (xmlNode *node)
 {
-  gboolean as_url = FALSE;
-  gboolean oab_url = FALSE;
-
   for (node = node->children; node; node = node->next)
     {
       if (ews_client_check_node (node, "ASUrl"))
-        as_url = TRUE;
-      else if (ews_client_check_node (node, "OABUrl"))
-        oab_url = TRUE;
-
-      if (as_url && oab_url)
-        break;
+        return TRUE;
     }
 
-  return as_url && oab_url;
+  return FALSE;
 }
 
 static gboolean
@@ -345,7 +335,7 @@ ews_client_autodiscover_response_cb (SoupSession *session, GAsyncResult *result,
       g_set_error (&error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED, /* TODO: more specific*/
-                   _("Failed to find ASUrl and OABUrl in autodiscover response"));
+                   _("Failed to find ASUrl in autodiscover response"));
       goto out;
     }
 
@@ -361,7 +351,7 @@ ews_client_autodiscover_response_cb (SoupSession *session, GAsyncResult *result,
           /* The callback (ie. this function) will be invoked after we
            * have returned to the main loop.
            */
-          g_cancellable_cancel (data->req_cancellable);
+          g_cancellable_cancel (data->cancellable);
         }
     }
 
@@ -542,7 +532,6 @@ goa_ews_client_autodiscover (GoaEwsClient        *self,
   data->session = soup_session_new ();
   soup_session_add_feature_by_type (data->session, SOUP_TYPE_AUTH_NTLM);
   data->accept_ssl_errors = accept_ssl_errors;
-  data->req_cancellable = g_cancellable_new ();
 
   if (cancellable != NULL)
     {
@@ -556,13 +545,13 @@ goa_ews_client_autodiscover (GoaEwsClient        *self,
   soup_session_send_and_read_async (data->session,
                                     data->msgs[0],
                                     G_PRIORITY_DEFAULT,
-                                    data->req_cancellable,
+                                    data->cancellable,
                                     (GAsyncReadyCallback)ews_client_autodiscover_response_cb,
                                     g_object_ref (task));
   soup_session_send_and_read_async (data->session,
                                     data->msgs[1],
                                     G_PRIORITY_DEFAULT,
-                                    data->req_cancellable,
+                                    data->cancellable,
                                     (GAsyncReadyCallback)ews_client_autodiscover_response_cb,
                                     g_object_ref (task));
 
